@@ -7,6 +7,9 @@
 
 import pytest
 
+from pymtl3.dsl.errors import UpdateBlockWriteError
+from pymtl3.passes.rtlir.util.test_utility import expected_failure
+
 from ...testcases import (
     CaseBits32ConstBitsToTmpVarComp,
     CaseBits32ConstIntToTmpVarComp,
@@ -14,11 +17,25 @@ from ...testcases import (
     CaseBits32MultiTmpWireComp,
     CaseBits32TmpWireAliasComp,
     CaseBits32TmpWireComp,
+    CaseUpdateffMixAssignComp,
 )
 from ..BehavioralTranslatorL2 import BehavioralTranslatorL2
 from .TestBehavioralTranslator import mk_TestBehavioralTranslator
 
 
+def run_test( case ):
+  m = case.DUT()
+  m.elaborate()
+  tr = mk_TestBehavioralTranslator(BehavioralTranslatorL2)(m)
+  tr.clear( m )
+  tr.translate_behavioral( m )
+  upblk_src = tr.behavioral.upblk_srcs[m]
+  decl_freevars = tr.behavioral.decl_freevars[m]
+  decl_tmpvars = tr.behavioral.decl_tmpvars[m]
+  assert upblk_src == case.REF_UPBLK
+  assert decl_freevars == case.REF_FREEVAR
+  assert decl_tmpvars == case.REF_TMPVAR
+
 @pytest.mark.parametrize(
   'case', [
     CaseBits32TmpWireComp,
@@ -30,14 +47,8 @@ from .TestBehavioralTranslator import mk_TestBehavioralTranslator
   ]
 )
 def test_generic_behavioral_L2( case ):
-  m = case.DUT()
-  m.elaborate()
-  tr = mk_TestBehavioralTranslator(BehavioralTranslatorL2)(m)
-  tr.clear( m )
-  tr.translate_behavioral( m )
-  upblk_src = tr.behavioral.upblk_srcs[m]
-  decl_freevars = tr.behavioral.decl_freevars[m]
-  decl_tmpvars = tr.behavioral.decl_tmpvars[m]
-  assert upblk_src == case.REF_UPBLK
-  assert decl_freevars == case.REF_FREEVAR
-  assert decl_tmpvars == case.REF_TMPVAR
+  run_test( case )
+
+def test_generic_behavioral_L2_updateff_mix_assign():
+  with expected_failure( UpdateBlockWriteError ):
+    run_test( CaseUpdateffMixAssignComp )
