@@ -6,6 +6,7 @@
 """Provide the level 1 SystemVerilog translator implementation."""
 
 from collections import deque
+from contextlib import contextmanager
 
 from pymtl3.datatypes import Bits, Bits32
 from pymtl3.passes.backends.generic.behavioral.BehavioralTranslatorL1 import (
@@ -94,7 +95,7 @@ class VBehavioralTranslatorL1( VBehavioralTranslatorL0, BehavioralTranslatorL1 )
     assert isinstance( rtype, rt.Const ), \
       f'{id_} freevar should be a constant!'
     assert isinstance( rtype.get_dtype(), rdt.Vector ), \
-      f'{id_} freevar should be a (list of) integer/BitStruct!'
+      f'{id_} freevar should be a (list of) integer/BitStruct at L1!'
     return s.rtlir_tr_const_decl( '__const__'+id_, rtype, array_type, dtype, obj )
 
 #-------------------------------------------------------------------------
@@ -163,6 +164,10 @@ class BehavioralRTLIRToVVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
       except ValueError:
         pass
 
+    # Are we inside the LHS of an assignemnt?
+
+    s.is_assign_LHS = False
+
     return s.visit( rtlir )
 
   def check_res( s, node, name ):
@@ -234,20 +239,26 @@ class BehavioralRTLIRToVVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
       target._top_expr = True
     node.value._top_expr = True
 
-    targets       = [ s.visit( target ) for target in node.targets ]
+    with s.register_assign_LHS():
+      targets     = [ s.visit( target ) for target in node.targets ]
+
     value         = s.visit( node.value )
     assignment_op = '<=' if not node.blocking else '='
     tplt = '{target} {assignment_op} {value};'
+
     return [ tplt.format(
       target = target, assignment_op = assignment_op, value = value
     ) for target in reversed(targets) ]
 
-  #-----------------------------------------------------------------------
-  # visit_If
-  #-----------------------------------------------------------------------
+  # register_assign_LHS
 
-  def visit_If( s, node ):
-    raise VerilogTranslationError( s.blk, node, "If not supported at L1" )
+  @contextmanager
+  def register_assign_LHS( s ):
+    assert not s.is_assign_LHS
+    s.is_assign_LHS = True
+    yield
+    assert s.is_assign_LHS
+    s.is_assign_LHS = False
 
   #-----------------------------------------------------------------------
   # visit_For
@@ -266,8 +277,9 @@ class BehavioralRTLIRToVVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
   #-----------------------------------------------------------------------
 
   def visit_Number( s, node ):
-    """Return a number in string without width specifier."""
-    return str( node.value )
+    """Return a number in string."""
+    nbits = node.Type.get_dtype().get_length()
+    return f"{nbits}'d{node.value}"
 
   #-----------------------------------------------------------------------
   # visit_Concat
@@ -289,14 +301,13 @@ class BehavioralRTLIRToVVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
     node.value._top_expr = True
 
     value = s.visit( node.value )
-    try:
-      target_nbits = int(node.nbits._value)
-    except AttributeError:
-      raise VerilogTranslationError( s.blk, node,
-        "new bitwidth of zero extension must be known at elaboration time!" )
+    target_nbits = node.nbits
     current_nbits = int(node.value.Type.get_dtype().get_length())
     padded_nbits = target_nbits - current_nbits
-    return f"{{ {{ {padded_nbits} {{ 1'b0 }} }}, {value} }}"
+    if padded_nbits == 0:
+      return value
+    else:
+      return f"{{ {{ {padded_nbits} {{ 1'b0 }} }}, {value} }}"
 
   #-----------------------------------------------------------------------
   # visit_SignExt
@@ -309,16 +320,14 @@ class BehavioralRTLIRToVVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
     node.value._top_expr = True
 
     value = s.visit( node.value )
-    try:
-      target_nbits = int(node.nbits._value)
-    except AttributeError:
-      raise VerilogTranslationError( s.blk, node,
-        "new bitwidth of sign extension must be known at elaboration time!" )
-
+    target_nbits = node.nbits
     current_nbits = int(node.value.Type.get_dtype().get_length())
     last_bit = current_nbits - 1
     padded_nbits = target_nbits - current_nbits
 
+    if padded_nbits == 0:
+      return value
+
     template = "{{ {{ {padded_nbits} {{ {value}[{last_bit}] }} }}, {value} }}"
     one_bit_template = "{{ {{ {padded_nbits} {{ {_value} }} }}, {value} }}"
 
@@ -353,6 +362,29 @@ class BehavioralRTLIRToVVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
     else:
       return template.format( **locals() )
 
+  #-----------------------------------------------------------------------
+  # visit_Truncate
+  #-----------------------------------------------------------------------
+  # To quote the LRM about the size casting operator:
+  # When changing the size, the cast shall return the value that a packed
+  # array type with a single [n-1:0] dimension would hold after being
+  # assigned the expression, where n is the cast size. The signedness shall
+  # pass through unchanged.
+  #
+  # This means if we translate the truncation into a size casting expression,
+  # the same semantics is preserved. This seems a better approach to me
+  # than slicing because iirc slicing over integer literals is an error
+  # in Verilator.
+
+  def visit_Truncate( s, node ):
+    nbits = node.nbits
+    value = s.visit( node.value )
+    dtype = node.value.Type.get_dtype()
+    if isinstance(dtype, rdt.Vector) and dtype.get_length() > nbits:
+      return f"{nbits}'({value})"
+    else:
+      return value
+
   #-----------------------------------------------------------------------
   # visit_Reduce
   #-----------------------------------------------------------------------
@@ -379,14 +411,10 @@ class BehavioralRTLIRToVVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
     nbits = node.nbits
     value = s.visit( node.value )
     if hasattr( node, "_value" ):
-      if isinstance( node._value, Bits ):
-        value = int(node._value)
-      else:
-        value = node._value
       # value could be larger than what nbits can hold because it might
       # be a negative number. Since verilator errors when that happens, we
       # need to manually truncate the integer.
-      value = int(Bits(nbits, value))
+      value = int(Bits(nbits, node._value))
       return f"{nbits}'d{value}"
 
     return f"{nbits}'( {value} )"
@@ -448,7 +476,11 @@ class BehavioralRTLIRToVVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
       # The base of this attribute node is the component 's'.
       # Example: s.out, s.STATE_IDLE
       # assert node.value.base is s.component
-      ret = attr
+      if isinstance( node.Type, rt.Const ) and isinstance( node.Type.get_dtype(), rdt.Vector ):
+        nbits = node.Type.get_dtype().get_length()
+        ret = f"{nbits}'( {attr} )"
+      else:
+        ret = attr
     else:
       raise VerilogTranslationError( s.blk, node,
           "sub-components are not supported at L1" )
@@ -477,8 +509,15 @@ class BehavioralRTLIRToVVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
             # format(node.value) )
 
       if isinstance( subtype, ( rt.Port, rt.Wire, rt.Const ) ):
-        # return f'{value}[{idx}]'
-        return s.process_unpacked_q( node, 
+        # Special case for situations like s.literal_int_array[0]
+        # TODO: verify that these cases have been captured by the
+        # constant extraction.
+        assert not isinstance( node.Type, rt.Const )
+        # if isinstance( node.Type, rt.Const ):
+        #   nbits = node.Type.get_dtype().get_length()
+        #   return f"{nbits}'({value}[{idx}])"
+
+        return s.process_unpacked_q( node,
             f'{value}[{idx}]', f'{value}{{}}[{idx}]' )
       else:
         # is this branch ever taken?
@@ -524,7 +563,9 @@ class BehavioralRTLIRToVVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
     # Regular [ lower : upper ] syntax
     else:
       if hasattr( node.upper, '_value' ):
-        upper = str( int( node.upper._value - Bits32(1) ) )
+        upper = str( int( node.upper._value - 1 ) )
+        nbits = node.upper.Type.get_dtype().get_length()
+        upper = f"{nbits}'d{upper}"
       else:
         upper = s.visit( node.upper ) + '-1'
 
@@ -549,7 +590,8 @@ class BehavioralRTLIRToVVisitorL1( bir.BehavioralRTLIRNodeVisitor ):
   #-----------------------------------------------------------------------
 
   def visit_FreeVar( s, node ):
-    return f'__const__{node.name}'
+    nbits = node.Type.get_dtype().get_length()
+    return f"{nbits}'( __const__{node.name} )"
 
   #-----------------------------------------------------------------------
   # visit_TmpVar
