@@ -24,6 +24,7 @@ from pymtl3.passes.rtlir.util.test_utility import do_test, expected_failure
 from pymtl3.passes.testcases import (
     CaseAssignMultiTargetComp,
     CaseAttributeSignalComp,
+    CaseBits64TruncInComp,
     CaseBitsArgsComp,
     CaseBitSelOutOfRangeComp,
     CaseClassdefComp,
@@ -80,6 +81,8 @@ from pymtl3.passes.testcases import (
     CaseStrComp,
     CaseStructBitsUnagreeableComp,
     CaseTmpComponentComp,
+    CaseTruncLargeNbitsComp,
+    CaseTruncVaribleNbitsComp,
     CaseTryExceptComp,
     CaseTryFinallyComp,
     CaseTupleComp,
@@ -115,11 +118,14 @@ def local_do_test( m ):
   try:
     ref = m._rtlir_test_ref
     for blk in m.get_update_blocks():
-      upblk = m._pass_behavioral_rtlir_gen.rtlir_upblks[ blk ]
+      upblk = m.get_metadata( BehavioralRTLIRGenL1Pass.rtlir_upblks )[ blk ]
       assert upblk == ref[ blk.__name__ ]
   except AttributeError:
     pass
 
+def test_L1_call_name( do_test ):
+  do_test( CaseNonNameCalledComp )
+
 #-------------------------------------------------------------------------
 # PyMTL type errors
 #-------------------------------------------------------------------------
@@ -133,21 +139,29 @@ def test_L1_concat_component( do_test ):
     do_test( CaseConcatComponentComp )
 
 def test_L1_zext_variable_nbits( do_test ):
-  with expected_failure( PyMTLTypeError, "not a constant number" ):
+  with expected_failure( PyMTLSyntaxError, "not a constant int or BitsN type" ):
     do_test( CaseZextVaribleNbitsComp )
 
 def test_L1_zext_small_nbits( do_test ):
-  with expected_failure( PyMTLTypeError, "not greater" ):
+  with expected_failure( PyMTLTypeError, "less than the bitwidth of the operand" ):
     do_test( CaseZextSmallNbitsComp )
 
 def test_L1_sext_variable_nbits( do_test ):
-  with expected_failure( PyMTLTypeError, "not a constant number" ):
+  with expected_failure( PyMTLSyntaxError, "not a constant int or BitsN type" ):
     do_test( CaseSextVaribleNbitsComp )
 
 def test_L1_sext_small_nbits( do_test ):
-  with expected_failure( PyMTLTypeError, "not greater" ):
+  with expected_failure( PyMTLTypeError, "less than the bitwidth of the operand" ):
     do_test( CaseSextSmallNbitsComp )
 
+def test_L1_trunc_variable_nbits( do_test ):
+  with expected_failure( PyMTLSyntaxError, "not a constant int or BitsN type" ):
+    do_test( CaseTruncVaribleNbitsComp )
+
+def test_L1_trunc_large_nbits( do_test ):
+  with expected_failure( PyMTLTypeError, "larger than the bitwidth of the operand" ):
+    do_test( CaseTruncLargeNbitsComp )
+
 def test_L1_wrong_attribute( do_test ):
   with expected_failure( PyMTLTypeError, "does not have attribute in_" ):
     do_test( CaseDroppedAttributeComp )
@@ -299,10 +313,6 @@ def test_L1_call_keyword_arg( do_test ):
   with expected_failure( PyMTLSyntaxError, "keyword argument" ):
     do_test( CaseKwArgsComp )
 
-def test_L1_call_name( do_test ):
-  with expected_failure( PyMTLSyntaxError, "function is not found" ):
-    do_test( CaseNonNameCalledComp )
-
 def test_L1_call_not_found( do_test ):
   with expected_failure( PyMTLSyntaxError, "function is not found" ):
     do_test( CaseFuncNotFoundComp )
