@@ -15,7 +15,7 @@ import copy
 import inspect
 import math
 
-import pymtl3.dsl as dsl
+from pymtl3 import dsl
 from pymtl3.datatypes import Bits, is_bitstruct_inst
 
 from ..errors import RTLIRConversionError
